<?php

	use Sprain\SwissQrBill as QrBill;		

	/**
	 * Dieses Module ermöglicht die Zahlungsart "Vorkasse"
	 */
	class wpsg_mod_prepayment extends wpsg_mod_basic
	{
		
		var $lizenz = 2;
		var $id = 1;

		const QRCODE_BOTH = '0';
        const QRCODE_EPC = '1';
        const QRCODE_SWISS = '2';
		
		/**
		 * Costructor
		 */
		public function __construct()
		{
			
			parent::__construct();
			
			$this->name = __('Vorkasse', 'wpsg');
			$this->group = __('Zahlungsarten', 'wpsg');
			$this->desc = __('Ermöglicht die Zahlungsart Vorkasse.', 'wpsg');
									
		} // public function __construct()
		
		public function install()
		{
			
			$this->shop->checkDefault('wpsg_mod_prepayment_bezeichnung', $this->name, false, true);
			$this->shop->checkDefault('wpsg_mod_prepayment_aktiv', '1');
			$this->shop->checkDefault('wpsg_mod_prepayment_qrcode', '1');
			$this->shop->checkDefault('wpsg_mod_prepayment_hint', __('Zahlen Sie die Bestellung mittels Überweisung. Der Betreff wird ihnen in der Bestellbestätigung mitgeteilt.', 'wpsg'), false, true);
			$this->shop->checkDefault('wpsg_mod_prepayment_subject', 'O%order_id% - K%kunde_id%');
			$this->shop->checkDefault('wpsg_mod_prepayment_mwst', '0');
			$this->shop->checkDefault('wpsg_mod_prepayment_mwstland', '0');
			
			$this->shop->checkDefault('wpsg_mod_prepayment_kinhaber', '%shopinfo_accountowner%');
			$this->shop->checkDefault('wpsg_mod_prepayment_bank', '%shopinfo_bankname%');
			$this->shop->checkDefault('wpsg_mod_prepayment_iban', '%shopinfo_iban%');
			$this->shop->checkDefault('wpsg_mod_prepayment_qriban', '%shopinfo_qriban%');
			$this->shop->checkDefault('wpsg_mod_prepayment_swift', '%shopinfo_bic%');
			$this->shop->checkDefault('wpsg_mod_prepayment_besr', '%shopinfo_besr%');
			
		} // public function install()
		
		public function settings_edit()
		{
			 		
			$this->render(WPSG_PATH_VIEW.'/mods/mod_prepayment/settings_edit.phtml');
			
		} // public function settings_edit()
		
		public function settings_save()
		{
			
		    $this->shop->update_option('wpsg_mod_prepayment_bezeichnung', $_REQUEST['wpsg_mod_prepayment_bezeichnung'], false, false, WPSG_SANITIZE_TEXTFIELD);
			$this->shop->update_option('wpsg_mod_prepayment_aktiv', $_REQUEST['wpsg_mod_prepayment_aktiv'], false, false, WPSG_SANITIZE_CHECKBOX);
			$this->shop->update_option('wpsg_mod_prepayment_qrcode', $_REQUEST['wpsg_mod_prepayment_qrcode'], false, false, WPSG_SANITIZE_CHECKBOX);
			$this->shop->update_option('wpsg_mod_prepayment_qrcode_country', $_REQUEST['wpsg_mod_prepayment_qrcode_country'], false, false, WPSG_SANITIZE_VALUES, [wpsg_mod_prepayment::QRCODE_BOTH, wpsg_mod_prepayment::QRCODE_EPC, wpsg_mod_prepayment::QRCODE_SWISS]);
			$this->shop->update_option('wpsg_mod_prepayment_mailattachment', $_REQUEST['wpsg_mod_prepayment_mailattachment'], false, false, WPSG_SANITIZE_CHECKBOX);
			$this->shop->update_option('wpsg_mod_prepayment_hint', $_REQUEST['wpsg_mod_prepayment_hint'], false, false, WPSG_SANITIZE_TEXTFIELD);
			$this->shop->update_option('wpsg_mod_prepayment_gebuehr', $_REQUEST['wpsg_mod_prepayment_gebuehr'], false, false, WPSG_SANITIZE_FLOAT);
			$this->shop->update_option('wpsg_mod_prepayment_mwst', $_REQUEST['wpsg_mod_prepayment_mwst'], false, false, WPSG_SANITIZE_TAXKEY);
			$this->shop->update_option('wpsg_mod_prepayment_mwstland', $_REQUEST['wpsg_mod_prepayment_mwstland'], false, false, WPSG_SANITIZE_CHECKBOX);
			$this->shop->update_option('wpsg_mod_prepayment_kinhaber', $_REQUEST['wpsg_mod_prepayment_kinhaber'], false, false, WPSG_SANITIZE_TEXTFIELD);
			$this->shop->update_option('wpsg_mod_prepayment_bank', $_REQUEST['wpsg_mod_prepayment_bank'], false, false, WPSG_SANITIZE_TEXTFIELD);
			$this->shop->update_option('wpsg_mod_prepayment_iban', $_REQUEST['wpsg_mod_prepayment_iban'], false, false, WPSG_SANITIZE_TEXTFIELD);
			$this->shop->update_option('wpsg_mod_prepayment_qriban', $_REQUEST['wpsg_mod_prepayment_qriban'], false, false, WPSG_SANITIZE_TEXTFIELD);
			$this->shop->update_option('wpsg_mod_prepayment_swift', $_REQUEST['wpsg_mod_prepayment_swift'], false, false, WPSG_SANITIZE_TEXTFIELD);
			$this->shop->update_option('wpsg_mod_prepayment_besr', $_REQUEST['wpsg_mod_prepayment_besr'], false, false, WPSG_SANITIZE_TEXTFIELD);
			$this->shop->update_option('wpsg_mod_prepayment_subject', $_REQUEST['wpsg_mod_prepayment_subject'], false, false, WPSG_SANITIZE_TEXTFIELD);
			
			$this->shop->addTranslationString('wpsg_mod_prepayment_bezeichnung', wpsg_sinput(WPSG_SANITIZE_TEXTFIELD, $_REQUEST['wpsg_mod_prepayment_bezeichnung']));
			$this->shop->addTranslationString('wpsg_mod_prepayment_hint', wpsg_sinput(WPSG_SANITIZE_TEXTFIELD, $_REQUEST['wpsg_mod_prepayment_hint']));
			
		} // public function settings_save()
		
		public function checkGeneralBackendError() 
		{ 

			if ($this->shop->get_option('wpsg_mod_prepayment_aktiv') === '1')
			{
				
				if (	!wpsg_isSizedString($this->shop->get_option('wpsg_mod_prepayment_bank')) ||
						!wpsg_isSizedString($this->shop->get_option('wpsg_mod_prepayment_iban')) ||
						!wpsg_isSizedString($this->shop->get_option('wpsg_mod_prepayment_swift')) || 
						!wpsg_isSizedString($this->shop->get_option('wpsg_mod_prepayment_kinhaber'))	)
				{
					
					$this->shop->addBackendError('nohspc_'.wpsg_translate(
						wpsg_translate(
							__('Die Zahlungsart "Vorkasse" ist aktiv, es wurden aber nur unvollständige Kontodaten angegeben. Überprüfen Sie die <a href="#1#">Moduleinstellungen</a>.', 'wpsg'),
							WPSG_URL_WP.'wp-admin/admin.php?page=wpsg-Admin&action=module&modul=wpsg_mod_prepayment'
						)
					));
					
				}
				
			}
			
		} // public function checkGeneralBackendError()
		
		public function addPayment(&$arPayment) { 
 
			if (!is_admin() && $this->shop->get_option('wpsg_mod_prepayment_aktiv') != '1') return;
			
			$arPayment[$this->id] = array(
				'id' => $this->id,
				'name' => __($this->shop->get_option('wpsg_mod_prepayment_bezeichnung'), 'wpsg'),
				'price' => $this->shop->get_option('wpsg_mod_prepayment_gebuehr'),
				'tax_key' => $this->shop->get_option('wpsg_mod_prepayment_mwst'),
				'mwst_null' => $this->shop->get_option('wpsg_mod_prepayment_mwstland'),
				'hint' => $this->shop->get_option('wpsg_mod_prepayment_hint'),
				'logo' => $this->shop->getRessourceURL('mods/mod_prepayment/gfx/logo_100x25.png')
			);
			 			
		} // public function addPayment(&$arShipping)
		 
		public function order_done(&$order_id, &$done_view) 
		{ 

			// Bestellungen mit 0 geben nix aus
			if ($done_view['basket']['sum']['preis_gesamt_brutto'] <= 0) return;
			
			if ($this->shop->view['basket']['checkout']['payment'] == $this->id)
			{
				
				$this->shop->view['wpsg_mod_prepayment']['subject'] = $this->get_subject($order_id);
				
				echo $this->shop->render(WPSG_PATH_VIEW.'/mods/mod_prepayment/order_done.phtml', false);

			}
						
		} // public function order_done(&$order_id)
		
		public function mail_payment() 
		{ 

			if ($this->shop->view['basket']['checkout']['payment'] != $this->id) return;
			
			$this->shop->view['mod_prepayment']['subject'] = $this->get_subject($this->shop->view['o_id']);
			
			if ($this->shop->htmlMail === true)
			{

				echo $this->shop->render(WPSG_PATH_VIEW.'/mods/mod_prepayment/mail_html.phtml');
				
			}
			else
			{
			
				echo $this->shop->render(WPSG_PATH_VIEW.'/mods/mod_prepayment/mail.phtml');
				
			}
			
		} // public function mail_payment()
				
		/**
		 * Ersetzt die Platzhalter im Betreff
		 */
		public function get_subject($order_id) {
			
			$this->shop->cache->clearOrderCache($order_id);
			
			$ret = $this->shop->replaceUniversalPlatzhalter($this->shop->get_option('wpsg_mod_prepayment_subject'), $order_id);
			
			return $ret;
			
		} // private function get_subject($order_id)

		public function getDownloadFile($order_id) {

            $file_path = $this->getFilePath($order_id);

            header('Content-Disposition: attachment; filename="'.$file_path.'"');

            return $file_path;

        } // public function getDownloadFile($order_id)

        /**
         * Startet den Download einer QR-Rechnung wird im Frontend wie im Backend verwendet
         */
        private function doDownload($order_id) {

            $file = $this->getDownloadFile($order_id);

            header("Cache-Control: no-cache, must-revalidate");
            header("Expires: Sat, 26 Jul 1997 05:00:00 GMT");
            header('Content-type: application/pdf');
            header('Content-Disposition: inline');

            die(file_get_contents($file));

        } // private function doDownload($order_id)
		
		/**
		 * Gibt einen Downloadlink für die Rechnung für die Verwendung im Frontend zurück
		 */
		public function getFrontendLink($order_id)
		{
			 
			$hash = md5($this->shop->get_option('wpsg_salt').'wpsg'.$order_id.'wpsg'.$this->shop->get_option('wpsg_salt'));
			
			$url = $this->shop->getDoneURL($order_id);
			$url .= ((strpos($url, '?') === false)?'?wpsg_action=wpsg_mod_prepayment_download':'&wpsg_action=wpsg_mod_prepayment_download');
			$url .= '&hash='.$hash.'&order_id='.$order_id;
			
			return $url;
			 
		} // public function getFrontendLink($order_id)

		/**
		 * @throws \Exception
		 */
		public function getFilePath($order_id) {

            $ym = date('Y/m');

			if ($this->shop->isMultiBlog()) $path = WP_CONTENT_DIR.'/'.WPSG_MB_UPLOADS.'/wpsg/wpsg_qrrechnungen/'.$ym.'/';
			else $path = WPSG_PATH_CONTENT.'uploads/wpsg/wpsg_qrrechnungen/'.$ym.'/';

			$path .= $order_id.'.pdf';

			\wpsg_ShopController::getShop()->protectDirectory(dirname($path));

			return $path;
			
		} // public function getFilePath()

		/**
		 * QR-Rechnung PDF für eMail erzeugen
		 */
		public function genQRBill(QrBill\QrBill $qrBill, $order_id) {

			require_once WPSG_PATH_LIB.'FPDF_1.81/fpdf.php';
			require_once WPSG_PATH_LIB.'FPDI_2.2.0/autoload.php';
			require_once WPSG_PATH_LIB.'wpsg_fpdf.class.php';

			if (!file_exists(WPSG_PATH_UPLOADS.'wpsg_qrrechnungen/')) mkdir(WPSG_PATH_UPLOADS.'wpsg_qrrechnungen/', 0755, true);

			$fpdf = new wpsg_fpdf('P', 'mm', 'A4');
			// if (ini_get('allow_url_fopen')) $fpdf = new wpsg_fpdf('P', 'mm', 'A4');
			// else {

			// 	$fpdf = new class('P', 'mm', 'A4') extends wpsg_fpdf {
			// 		use \Fpdf\Traits\MemoryImageSupport\MemImageTrait;
			// 	};

			// }

			$fpdf->AddPage();

			$output = new QrBill\PaymentPart\Output\FpdfOutput\FpdfOutput($qrBill, 'en', $fpdf);
			$output->setPrintable(false)->getPaymentPart();

			$path = $this->getFilePath($order_id);
			$fpdf->Output($path, 'F');

		} // public function genQRBill($qrBill)

		/**
		 * Funktion zum Generieren des Base64-Strings aus dem Schweizer QR-Code
		 */
		private function generateBase64QrCode(QrBill\QrBill $qrBill) {
 
	 $tmpfname = tempnam("/tmp", "swissqrcode").'.png';
	 $qrCode = $qrBill->getQrCode(QrBill\QrCode\QrCode::FILE_FORMAT_PNG);
	 $qrCode->writeFile($tmpfname);
	 $base64QrCode = base64_encode(file_get_contents($tmpfname));
 
			return $base64QrCode;
			
		} // public function generateBase64QrCode($qrBill)
		
		/**
		 * QRCode für eMail erzeugen
		 */
		public function genQRCode($order_id, $subject, $betrag, $size, $out, $target, $data = null) {
			
			require_once WPSG_PATH_LIB.'phpqrcode/qrlib.php';
			require_once WPSG_PATH_LIB.'phpgirocode.class.php';
			require_once WPSG_PATH_LIB.'vendor/autoload.php';
			
            if (!file_exists(WPSG_PATH_UPLOADS.'wpsg_girocode/')) mkdir(WPSG_PATH_UPLOADS.'wpsg_girocode/', 0755, true);
			if (!file_exists(WPSG_PATH_UPLOADS.'wpsg_swissqrbill/')) mkdir(WPSG_PATH_UPLOADS.'wpsg_swissqrbill/', 0755, true);

			if ($target == wpsg_mod_prepayment::QRCODE_BOTH) {

				$ret = []; // return array
				$kundendaten = $data['checkout']; 

				// EPC - QR-Code
				$fname1 = WPSG_PATH_UPLOADS.'wpsg_girocode/'.$order_id.'.png';
								
					$phpGiroCode = new PhpGirocode(); 
					$phpGiroCode->setBIC(wpsg_getStr($this->shop->get_option('wpsg_shopdata_bank_bic')));
					$phpGiroCode->setIBAN(wpsg_getStr($this->shop->get_option('wpsg_shopdata_bank_iban'))); 
					$phpGiroCode->setReciver(wpsg_getStr($this->shop->get_option('wpsg_shopdata_bank_owner')));
					$phpGiroCode->setAmount($betrag); 
					$phpGiroCode->setText($subject); 
					$phpGiroCode->setSize($size);
					
				if ($out == PhpGirocode::OUTPUT_BROWSER) $phpGiroCode->generate(PhpGirocode::OUTPUT_BROWSER, $fname1);
				else if ($out == PhpGirocode::OUTPUT_FILE) {
				
					$phpGiroCode->generate(PhpGirocode::OUTPUT_FILE, $fname1);
					$fname2 = WPSG_URL_WP.'?wpsg_action=wpsg_getGiroCode&order_id='.$order_id;
					$data1 = array(
						'title' => 'Girocode',
						'value' => '<img style="max-width:120px;" width="120" height="120" src="'.$fname2.'" />'
					);
					$ret[] = $data1;

				} else if ($out == PhpGirocode::OUTPUT_BASE64) {
					
					$tmpfname = @tempnam("/tmp", "phpgirocode");
					$qrCode = array(
						'title' => 'Girocode',
						'hash' => $phpGiroCode->generate(PhpGirocode::OUTPUT_BASE64, $tmpfname),
						'referenceNumber' => $referenceNumber??'',
						'additionalInformation' => $subject,
						'customerdata' => $kundendaten, 
						'invoiceAmount' => $betrag,
						'order_id' => $order_id
					);		
					$ret[] = $qrCode;		
					
				} else $ret[] = $phpGiroCode->generate(PhpGirocode::OUTPUT_TEST, false);
			
				// SWISS - QR-Code
				$fname3 = WPSG_PATH_UPLOADS.'wpsg_swissqrbill/'.$order_id.'.png';
				$oCountryB = wpsg_country::getInstance($this->shop->get_option('wpsg_shopdata_country'));
				$oCountryC = wpsg_country::getInstance($kundendaten['land']); 
                $besrId = '';
				if (!empty($this->shop->get_option('wpsg_shopdata_bank_besr'))) $besrId = $this->shop->get_option('wpsg_shopdata_bank_besr'); // should be empty if bank = "Postfinance", see also tooltip

					$qrBill = QrBill\QrBill::create();
					$qrBill->setCreditor(
						QrBill\DataGroup\Element\CombinedAddress::create(
							wpsg_getStr($this->shop->get_option('wpsg_shopdata_name')), 
							$this->shop->get_option('wpsg_shopdata_street'), 
							$this->shop->get_option('wpsg_shopdata_zip').' '.wpsg_getStr($this->shop->get_option('wpsg_shopdata_city')), 
							($oCountryB->getShorttext()??'')
						)
					);
					$qrBill->setCreditorInformation(
						QrBill\DataGroup\Element\CreditorInformation::create(
							$this->shop->get_option('wpsg_shopdata_bank_qriban') 
						)
					);
					$qrBill->setUltimateDebtor(
						QrBill\DataGroup\Element\StructuredAddress::createWithStreet(
							wpsg_getStr($kundendaten['vname']).' '.wpsg_getStr($kundendaten['name']), 
							wpsg_getStr($kundendaten['strasse']), 
							$kundendaten['nr'], 
							$kundendaten['plz'], 
							wpsg_getStr($kundendaten['ort']),
							$oCountryC->getShorttext()??''
						)
					);
					$qrBill->setPaymentAmountInformation(
						QrBill\DataGroup\Element\PaymentAmountInformation::create(
							wpsg_getStr($this->shop->get_option('wpsg_currency')), 
							$betrag 
						)
					);
					$referenceNumber = QrBill\Reference\QrPaymentReferenceGenerator::generate(
						$besrId, 
						$kundendaten['onr']
					);
					$qrBill->setPaymentReference(
						QrBill\DataGroup\Element\PaymentReference::create(
							QrBill\DataGroup\Element\PaymentReference::TYPE_QR,
							$referenceNumber
						)
					);
					$qrBill->setAdditionalInformation(
						QrBill\DataGroup\Element\AdditionalInformation::create(
							$subject ?? ''
						)
					);
					
				if ($out == PhpGirocode::OUTPUT_BROWSER) {

					header('Content-Type: image/svg'); 
					$qrBill->getQrCode(QrBill\QrCode\QrCode::FILE_FORMAT_SVG);

				} else if ($out == PhpGirocode::OUTPUT_FILE) {		

					$qrBill->getQrCode()->writeFile($fname3);
					$fname4 = WPSG_URL_WP.'?wpsg_action=wpsg_getQrInvoice&order_id='.$order_id;
					$data2 = array(
						'title' => 'QR-Rechnung',
						'value' => '<img style="max-width:120px;" width="120" height="120" src="'.$fname4.'" />'
					);
					$ret[] = $data2;

				} else if ($out == PhpGirocode::OUTPUT_BASE64) {

					// wpsg_debug($qrBill->getViolations());
 
					$qrCode = array(
						'title' => 'QR-Rechnung',
						'hash' => $this->generateBase64QrCode($qrBill),
						'referenceNumber' => $referenceNumber,
						'additionalInformation' => $subject,
						'customerdata' => $kundendaten, 
						'invoiceAmount' => $betrag,
						'filepath' => $this->getFilePath($order_id).'/'.$order_id.'.pdf',
						'order_id' => $order_id
					);
					$ret[] = $qrCode; 

					if (!file_exists($this->getFilePath($order_id).'/'.$order_id.'.pdf')) $this->genQRBill($qrBill, $order_id);

				}

			} else if ($target == wpsg_mod_prepayment::QRCODE_EPC) {

				$kundendaten = $data['checkout']; 

				$fname1 = WPSG_PATH_UPLOADS.'wpsg_girocode/'.$order_id.'.png';
							
					$phpGiroCode = new PhpGirocode(); 
					$phpGiroCode->setBIC(wpsg_getStr($this->shop->get_option('wpsg_shopdata_bank_bic'))); 
					$phpGiroCode->setIBAN(wpsg_getStr($this->shop->get_option('wpsg_shopdata_bank_iban'))); 
					$phpGiroCode->setReciver(wpsg_getStr($this->shop->get_option('wpsg_shopdata_bank_owner'))); 
					$phpGiroCode->setAmount($betrag); 
					$phpGiroCode->setText($subject); 
					$phpGiroCode->setSize($size);
				
				if ($out == PhpGirocode::OUTPUT_BROWSER) $phpGiroCode->generate(PhpGirocode::OUTPUT_BROWSER, $fname1);
				else if ($out == PhpGirocode::OUTPUT_FILE) {

					$phpGiroCode->generate(PhpGirocode::OUTPUT_FILE, $fname1);
					$fname2 = WPSG_URL_WP.'?wpsg_action=wpsg_getGiroCode&order_id='.$order_id;
					$data = array(
						'title' => 'Girocode',
						'value' => '<img style="max-width:120px;" width="120" height="120" src="'.$fname2.'" />'
					);
					$ret = $data;

				} else if ($out == PhpGirocode::OUTPUT_BASE64) {

					$tmpfname = @tempnam("/tmp", "phpgirocode");	
					$qrCode = array(
						'title' => 'Girocode',
						'hash' => $phpGiroCode->generate(PhpGirocode::OUTPUT_BASE64, $tmpfname),
						'referenceNumber' => $referenceNumber??'',
						'additionalInformation' => $subject,
						'customerdata' => $kundendaten, 
						'invoiceAmount' => $betrag,
						'order_id' => $order_id
					);		
					$ret = $qrCode;		

				} else $ret = $phpGiroCode->generate(PhpGirocode::OUTPUT_TEST, false);

			} else if ($target == wpsg_mod_prepayment::QRCODE_SWISS) {

				$kundendaten = $data['checkout']; 

				$fname3 = WPSG_PATH_UPLOADS.'wpsg_swissqrbill/'.$order_id.'.png';
				$oCountryB = wpsg_country::getInstance($this->shop->get_option('wpsg_shopdata_country'));
				$oCountryC = wpsg_country::getInstance($kundendaten['land']); 

				$besrId = '';
				if (!empty($this->shop->get_option('wpsg_shopdata_bank_besr'))) $besrId = $this->shop->get_option('wpsg_shopdata_bank_besr'); // should be empty if bank = "Postfinance", see also tooltip

					$qrBill = QrBill\QrBill::create();
					$qrBill->setCreditor(
						QrBill\DataGroup\Element\CombinedAddress::create(
							wpsg_getStr($this->shop->get_option('wpsg_shopdata_name')), 
							$this->shop->get_option('wpsg_shopdata_street'), 
							$this->shop->get_option('wpsg_shopdata_zip').' '.wpsg_getStr($this->shop->get_option('wpsg_shopdata_city')), 
							wpsg_getStr($oCountryB->getShorttext()) 
						)
					);
					$qrBill->setCreditorInformation(
						QrBill\DataGroup\Element\CreditorInformation::create(
							$this->shop->get_option('wpsg_shopdata_bank_qriban') 
						)
					);
					$qrBill->setUltimateDebtor(
						QrBill\DataGroup\Element\StructuredAddress::createWithStreet(
							wpsg_getStr($kundendaten['vname']).' '.wpsg_getStr($kundendaten['name']), 
							wpsg_getStr($kundendaten['strasse']), 
							$kundendaten['nr'], 
							$kundendaten['plz'], 
							wpsg_getStr($kundendaten['ort']),
							($oCountryC->getShorttext()??'')
						)
					);
					$qrBill->setPaymentAmountInformation(
						QrBill\DataGroup\Element\PaymentAmountInformation::create(
							wpsg_getStr($this->shop->get_option('wpsg_currency')), 
							$betrag 
						)
					);
					$referenceNumber = QrBill\Reference\QrPaymentReferenceGenerator::generate(
						$besrId, 
						$kundendaten['onr']
					);
					$qrBill->setPaymentReference(
						QrBill\DataGroup\Element\PaymentReference::create(
							QrBill\DataGroup\Element\PaymentReference::TYPE_QR,
							$referenceNumber
						)
					);
					$qrBill->setAdditionalInformation(
						QrBill\DataGroup\Element\AdditionalInformation::create(
							$subject
						)
					);
				

				if ($out == PhpGirocode::OUTPUT_BROWSER) {

					header('Content-Type: image/svg'); 
					$qrBill->getQrCode(QrBill\QrCode\QrCode::FILE_FORMAT_SVG);

				} else if ($out == PhpGirocode::OUTPUT_FILE) {	

					$qrBill->getQrCode()->writeFile($fname3);
					$fname4 = WPSG_URL_WP.'?wpsg_action=wpsg_getQrInvoice&order_id='.$order_id;
					$data = array(
						'title' => 'QR-Rechnung',
						'value' => '<img style="max-width:120px;" width="120" height="120" src="'.$fname4.'" />'
					);
					$ret = $data;

				} else if ($out == PhpGirocode::OUTPUT_BASE64) {

					// wpsg_debug($qrBill->getViolations());

					$qrCode = array(
						'title' => 'QR-Rechnung',
						'hash' => $this->generateBase64QrCode($qrBill),
						'referenceNumber' => $referenceNumber,
						'additionalInformation' => $subject,
						'customerdata' => $kundendaten, 
						'invoiceAmount' => $betrag,
						'filepath' => $this->getFilePath($order_id).'/'.$order_id.'.pdf',
						'order_id' => $order_id
					);		
					$ret = $qrCode;

					if (!file_exists($this->getFilePath($order_id).'/'.$order_id.'.pdf')) $this->genQRBill($qrBill, $order_id);

				}

			}
			
			return $ret;
			
		} // private function genQRCode($betrag, $subject, $size, $out, $target, $data)
		
		public function template_redirect() {
			
			if (wpsg_isSizedString($_REQUEST['wpsg_action'], 'wpsg_getGiroCode')) {
				
				header("Content-Type: image/png");
				
				readfile(WPSG_PATH_UPLOADS.'wpsg_girocode/'.intval(wpsg_sinput("key", $_REQUEST['order_id'])).'.png');
				
			}

			if (wpsg_isSizedString($_REQUEST['wpsg_action'], 'wpsg_getQrInvoice')) {
				
				header("Content-Type: image/png");
				
				readfile(WPSG_PATH_UPLOADS.'wpsg_swissqrbill/'.intval(wpsg_sinput("key", $_REQUEST['order_id'])).'.png');
				
			}

			if (wpsg_isSizedString($_REQUEST['wpsg_action'], 'wpsg_mod_prepayment_download'))
			{
				
				$hash = md5($this->shop->get_option('wpsg_salt').'wpsg'.$_REQUEST['order_id'].'wpsg'.$this->shop->get_option('wpsg_salt'));
				
				if ($hash != $_REQUEST['hash']) die(__('Kein Zugriff', 'wpsg'));
					
				$this->doDownload($_REQUEST['order_id']);
				
			}	
			
		}
		
	} // class wpsg_mod_prepayment extends wpsg_mod_basic

?>